* Exercises.sas
* Purpose:  Exercises for the Hands-on-Workshop
*           Five Ways to Create Macro Variables: 
*           A Short Introduction to the Macro Language   
* 
* Written by:
*  Art Carpenter
*  April 3, 2017
*  
*
* Notes:    A set of potential solutions can be found in SOLUTIONS.SAS
*           in this same directory.
*
* Modified:
*
*
*********************************************************;

************************;
* %LET
*  Exercise 1;
*     Create a macro variable &DSN which contains the 
*     data set name SASHELP.CLASS;
*;
%let xxxx = xxxx;


************************;
* Using the macro variable
*  Exercise 2;
*     Use the macro variable in the following PROC PRINT.
*;
proc print data=xxxx;
title1 'The data is xxxx';
run;


************************;
* %PUT
*  Exercise 3;
*     Use the %PUT statement and the macro variable write out 
*         The data set is sashelp.class 
*;
%put The data set is xxxx;


************************;
* %PUT
*  Exercise 4;
*     Use the %PUT statement to list all user defined macro variables.
*;
%put xxxxx;

************************;
* Positional Parameters
*  Exercise 5;
*     Supply macro variable names and the macro call
*     so that %LOOK will print 10 observations of SASHELP.CLASS.
*     Do not reuse the macro variable name from Ex 1.
*;
%MACRO LOOK(xxxx,yyyy);
   PROC CONTENTS DATA=xxxx;
      TITLE "DATA SET xxxx";
      RUN;

   PROC PRINT DATA=xxxx (OBS=yyyy); 
      TITLE2 "FIRST yyyy OBSERVATIONS";
      RUN;
%MEND LOOK;
%look(aaaa,bbbb)


************************;
* Named or Keyword Parameters
*  Exercise 6;
*     Convert the Ex 5 macro to use Named parameters.
*     Assign a default value to at least one parameter.
*;
%MACRO LOOK(dsn,obs);
   PROC CONTENTS DATA=&dsn;
      TITLE "DATA SET &dsn";
      RUN;

   PROC PRINT DATA=&dsn (OBS=&obs); 
      TITLE2 "FIRST &obs OBSERVATIONS";
      RUN;
%MEND LOOK;
%look(sashelp.class,10)


************************;
* Using the %DO loop
*  Exercise 7;
*     Write a macro that will break up the SASHELP.CLASS 
*     data set into one data set for each age.  Pass the lowest
*     and highest ages as macro parameters.
*;
%macro dsbyage(start=10, stop=18);
   %do xxxx = xxxx %to xxxx;
      data agexxxx;
         set sashelp.class;
         where age = xxxx;
         run;
   %end;
%mend dsbyage;
%dsbyage(start=12, stop=15)


************************;
* Using PROC SQL
*  Exercise 8;
*     Using PROC SQL write a macro that counts the number of 
*     students of a given AGE and places the count in a macro variable.
*;
%macro agecnt(age=12);
   proc sql noprint;
      select count(xxxx)
         into : yyyy
            from sashelp.class
               where age=&xxxx;
      quit;
   %put The count for age &xxxx is &yyyy;
%mend agecnt;
%agecnt(xxxx=14)


************************;
* Using PROC SQL
*  Exercise 9;
*     Using PROC SQL, write a macro that places a list of the unique 
*     values of a variable whose name is passed as a parameter into 
*     a single macro variable.
*;
%macro vlist(vname=name);
   proc sql noprint;
      select distinct(&xxxx)
         into : yyyy separated by ' '
            from sashelp.class;
      quit;
   %put &xxxx takes on the values of &yyyy;
%mend vlist;
%vlist(vname=age)


************************;
* Using CALL SYMPUTX
*  Exercise 10;
*     Use a DATA step to store the maximum value of a 
*     numeric DATA step variable in a macro variable.
*;
%macro maxval(vname=weight);
   data maxvalue;
      set sashelp.class;
      retain maxval .;
      maxval = max(maxval,&vname);
      call symputx(xxxx,xxxx,'l');
      run;
   %put The maximum value of xxxx is xxxx;
%mend maxval;
%maxval(vname=age)


************************;
* Using CALL SYMPUTX
*  Exercise 11;
*     Use a DATA step and store each name into a separate  
*     numbered macro variable.  Use a %DO loop and a %PUT to
*     step through the list one at a time.
*;
%macro namelist;
   data _null_;
      set sashelp.class;
      by name;
      if first.name then do;
         cnt+1;
         call symputx('xxxx'||left(put(cnt,5.)),name);
         call symputx('xxxx',left(put(cnt,5.))); 
      end;
      run;

   %put num  Name;
   %do xxxx = 1 %to &xxxx;
      %put &xxxx    &&xxxx&xxxx;
   %end;
%mend namelist;
%namelist
