* Solutions.sas
* Purpose:  These are possible solutions for the exercises in EXERCISES.SAS   
* 
* Written by:
*  Art Carpenter
*  April 3, 2017
*  
*
* Notes:
*
* Modified:
*
*
*********************************************************;


************************;
* %LET
*  Exercise 1;
*     Create a macro variable &DSN which contains the 
*     data set name SASHELP.CLASS;
*;
%let dsn = sashelp.class;


************************;
* Using the macro variable
*  Exercise 2;
*     Use the macro variable in the following PROC PRINT.
*;
proc print data=&dsn;
title1 "the data is &dsn";
run;


************************;
* %PUT
*  Exercise 3;
*     Use the %PUT statement and the macro variable write out 
         The data set is sashelp.class 
*;
%put The data set is &dsn;


************************;
* %PUT
*  Exercise 4;
*     Use the %PUT statement to list all user defined macro variables.
*;
%put _user_;


************************;
* Positional Parameters
*  Exercise 5;
*     Supply macro variable names and the macro call
*     so that %LOOK will print 10 observations of SASHELP.CLASS
*;
%MACRO LOOK(dset, obs);
   PROC CONTENTS DATA=&dset;
      TITLE "DATA SET &dset";
      RUN;

   PROC PRINT DATA=&dset (OBS=&obs); 
      TITLE2 "FIRST &obs OBSERVATIONS";
      RUN;
%MEND LOOK;
%look(sashelp.class, 10)


************************;
* Named or Keyword Parameters
*  Exercise 6;
*     Convert the Ex 5 macro to use Named parameters.
*     Assign a default value to at least one parameter.
*;
%MACRO LOOK(dset=, obs=5);
   PROC CONTENTS DATA=&dset;
      TITLE "DATA SET &dset";
      RUN;

   PROC PRINT DATA=&dset (OBS=&obs); 
      TITLE2 "FIRST &obs OBSERVATIONS";
      RUN;
%MEND LOOK;
%look(dset=sashelp.class, obs=10)


************************;
* Using the %DO loop
*  Exercise 7;
*     Write a macro that will break up the SASHELP.CLASS 
*     data set into one data set for each age.  Pass the lowest
*     and highest ages as macro parameters.
*;
%macro dsbyage(start=10, stop=18);
   %do age = &start %to &stop;
      data age&age;
         set sashelp.class;
         where age = &age;
         run;
   %end;
%mend dsbyage;
%dsbyage(start=12, stop=15)


************************;
* Using PROC SQL
*  Exercise 8;
*     Using PROC SQL write a macro that counts the number of 
*     students of a given age and places the count in a macro variable.
*;
%macro agecnt(age=12);
   proc sql noprint;
      select count(age)
         into : agecnt
            from sashelp.class
               where age=&age;
      quit;
   %put The count for age &age is &agecnt;
%mend agecnt;
%agecnt(age=14)


************************;
* Using PROC SQL
*  Exercise 9;
*     Using PROC SQL, write a macro that places a list of the unique 
*     values of a variable whose name is passed as a parameter into 
*     a single macro variable.
*;
%macro vlist(vname=name);
   proc sql noprint;
      select distinct(&vname)
         into : list&vname separated by ' '
            from sashelp.class;
      quit;
   %put The values of &vname are stored in list&vname and take on the values of &&list&vname;
%mend vlist;
%vlist(vname=age)


************************;
* Using CALL SYMPUTX
*  Exercise 10;
*     Use a DATA step to store the maximum value of a 
*     numeric DATA step variable in a macro variable.
*;
%macro maxval(vname=weight);
   data maxvalue;
      set sashelp.class;
      retain maxval .;
      maxval = max(maxval,&vname);
      call symputx('maximum',maxval,'l');
      run;
   %put The maximum value of &vname is &maximum;
%mend maxval;
%maxval(vname=age)


************************;
* Using CALL SYMPUTX
*  Exercise 11;
*     Use a DATA step and store each name into a separate  
*     numbered macro variable.  Use a %DO loop and a %PUT to
*     step through the list one at a time.
*;
%macro namelist;
   data _null_;
      set sashelp.class;
      by name;
      if first.name then do;
         cnt+1;
         call symputx('name'||left(put(cnt,5.)),name);
         call symputx('namecnt',left(put(cnt,5.))); 
      end;
      run;

   %put num  Name;
   %do namenum = 1 %to &namecnt;
      %put &namenum    &&name&namenum;
   %end;

   %put The list of user defined Macro vars is:;
   %put _user_;
%mend namelist;
%namelist
